/******************************************************************************/
/*                                                                            */
/* Name:   INSTALL.REX - CICS Information Center installation command         */
/* Author: David Page                                                         */
/* Date:   14th April 2000                                                    */
/* Function:                                                                  */
/*         This REXX command installs the CICS Information Center to a        */
/*         target directory.  It will eventually be replaced by an            */
/*         InstallShield wizard once I have found out how to create one.      */
/*                                                                            */
/******************************************************************************/

/* Find out where we are                                                      */
Parse source . . command
If left(command,1)='"'
  then parse var command '"' command '"'
  else parse var command command .

/* Get any parameters that have been specified in the invocation              */
Parse arg options
installType     = GetParameter(options,'type')
sourceDirectory = GetParameter(options,'source')
targetDirectory = GetParameter(options,'target')
webServer       = GetParameter(options,'server')
webDirectory    = GetParameter(options,'directory')
portNumber      = GetParameter(options,'port')

/* Find out where the source is                                               */
If sourceDirectory=''
  then do
    sourceDirectory = truncate(command,length(filespec('name',command)))
    If right(sourceDirectory,1)<>'\'
      then sourceDirectory = sourceDirectory'\'
  end

/* Set local variables                                                        */
eol = '0D0A'x

/* Find out if NetQuestion has been installed                                 */
imninstsrv = value('IMNINSTSRV',,'ENVIRONMENT')
If imninstsrv=''
  then do
    Say 'NetQuestion has not been installed on this machine.'
    reply = GetYesNo('Do you want to install it now?')
    If reply='Y'
      then do
        sourceDirectory'nq\v55\setup.exe'
        Call lineout , 'After you have installed NetQuestion, you must reboot your system'
        Call lineout , 'before installing the Information Center'
        exit 1
      end
      else do
        Call lineout , 'You must install NetQuestion and rebooot your system before'
        Call lineout , 'installing the Information Center if you want to use the search'
        Call lineout , 'functions.'
      end
  end

/******************************************************************************/
/* Check all the parameters and get the values for any that are missing       */
/******************************************************************************/
If installType="" | wordpos(translate(installType),'1 2 STANDALONE SERVER')=0
  then do
    /* Tell the user about the Information Center                             */
    Call lineout , 'You can use the CICS Information Center directly from the CD ROM on which it'
    Call lineout , 'is supplied without installing it.  To do this start your internet browser and'
    Call lineout , 'go to the root directory on the CD.  The initial file is infoctr.html.  When'
    Call lineout , 'working directly from the CD the search functions provided by NetQuestion are'
    Call lineout , 'not available.'
    Call lineout , ''
    Call lineout , 'Alternatively, you can install the CICS Information Center in one of three'
    Call lineout , 'ways:'
    Call lineout , ''
    Call lineout , '(1)  Full Stand-alone installation:  You can install the whole information '
    Call lineout , '     center on a stand-alone machine.  This will require about 240MB of space '
    Call lineout , '     on your hard disk.'
    Call lineout , '     '
    Call lineout , '(2)  Server install:  You can install the CICS Information Center on a server'
    Call lineout , '     machine to allow it to be accessed by other machines.'

    /* Ask what type of installation is required                              */
    do forever
      Call lineout , ''
      installType = GetReply('Please enter the type of installation you want (or 0 for none):')
      If installType='' | wordpos(installType,'0 1 2')=0
        then do
          Call lineout , 'ERROR  - the installation type must be a digit between 0 and 2.'
          iterate
        end
      If installType=0
        then do
          Call lineout , 'O.K. - ciao'
          exit 2
        end
      leave
    end
  end
If datatype(installType)<>'NUM'
  then installType = wordpos(translate(installType),'STANDALONE SERVER')
installType = word('StandAlone Server',installType)

/******************************************************************************/
/*                                                                            */  
/* There are six variables that are set to control the installation and the   */
/* locations of various files.  The six variables are:                        */
/*                                                                            */
/*   targetDirectory - the full DOS path name of the directory where the      */
/*                 Information Center will be installed.                      */
/*                                                                            */
/*   webDirectory - for a server installation this is the full DOS path name  */
/*                 of the root directory of the Web server.  For other        */
/*                 types of installation it is the empty string.              */
/*                                                                            */
/*   webRelativeDirectory - for a server installation this is the relative    */
/*                 path from the Web server root directory (contained in      */
/*                 webDirectory) to the target directory (which must be       */
/*                 beneath the web directory).                                */
/*                                                                            */
/*   webServer - for a server installation, the name of the web server        */
/*                                                                            */
/*   NQserver - the name and port number of the NetQuestion server.           */
/*   NQdirectory - the protocol and name of the directory containing the data */
/*                 files used by NetQuestion.                                 */
/*                                                                            */
/* In addition to these variables, the following values are set in the file   */
/* install.js                                                                 */
/*                                                                            */
/*   webDirectory                                                             */
/*   webRelativeDirectory                                                     */
/*   NQserver                                                                 */
/*   indexSuffix - used as a suffix to each of the NetQuestion index names.   */
/*                                                                            */
/*                                                                            */
/* Examples:                                                                  */
/*                                                                            */
/* (1) Full standalone installation                                           */
/*        targetDirectory        D:\ic99\                                     */
/*        webDirectory           <null>                                       */
/*        webRelativeDirectory   ic99\                                        */
/*        webServer              <null>                                       */
/*        NQserver               http://localhost:49213                       */
/*        NQdirectory            D:/ic99/html                                 */
/*                                                                            */
/* (2) Server installation                                                    */
/*        targetDirectory        C:\PROGRA~1\APACHE~1\Apache\htdocs\ic99\     */
/*        webDirectory           C:\PROGRA~1\APACHE~1\Apache\htdocs           */
/*        webRelativeDirectory   ic99\                                        */
/*        webServer              http://cicsid.hursley.ibm.com                */
/*        NQserver               http://cicsid.hursley.ibm.com:49213          */
/*        NQdirectory            http://cicsid.hursley.ibm.com/ic99/html/     */
/*                                                                            */
/******************************************************************************/

/* If this is a server installation then get the name of the server           */
If installType='Server'
  then do
    If webServer=''
      then webServer = GetReply('Please enter the name of the (HTML) web server (e.g. cicsid.hursley.ibm.com): ')
    webServer = 'http://'webServer

    do forever    
      If webDirectory=''
        then webDirectory = GetReply('Please enter the complete drive and path of the root directory of the' || eol ,
                                     'web server:')
      If ''<>directory(webDirectory) then leave
      If 'N'=GetYesNo('ERROR - directory' webDirectory 'does not exist.  You must enter the name of the root' || eol ,
                           '       directory of the web server.  Do you want to try again?')
        then exit 3
    end
    trash = directory(sourceDirectory)
    If right(webDirectory,1)<>'\'
      then webDirectory = webDirectory'\'

    If targetDirectory=''
      then prompt = 'Please enter the name of the directory where you want the Information' || eol ,
                 || 'Center to be installed (relative to the root directory of the web server):'
  end
  else do
    webDirectory = ''
    If targetDirectory=''
      then prompt = 'Please enter the full pathname of the directory where you want the Information' || eol ,
                 || 'Center to be installed (include the drive and path):'
  end

/******************************************************************************/
/*  Try to find out where the user wants to install the Information Center    */
/******************************************************************************/
do until targetDirectory<>''

  /* Keep prompting until he tells us where or until he gives up              */
  do while targetDirectory=''
    /* Get the name of the target directory                                   */
    targetDirectory = GetReply(prompt)
    prompt = ''

    /* Check that a directory has been specified                              */
    If targetDirectory=''
      then do
        If 'Y'=GetYesNo('You didn''t specify a directory.  Do you want to give up?')
          then exit 4
        prompt = 'O.K. - enter the name of the directory:'
        Iterate
      end
  end

  /* Check that the directory exists                                          */
  target = directory(webDirectory || targetDirectory)
  If target=''
    then do
      Call lineout , 'Directory "'webDirectory || targetDirectory'" does not exist.'
      If 'N'=GetYesNo('Do you want it to be created? y/n: ')
        then do
          If 'N'=GetYesNo('Do you want to specify another directory?')
            then exit 5
          prompt = 'O.K. - enter the name of the directory:'
          targetDirectory = ''
          Iterate
        end
      /* Try to create the directory                                          */
      rc = MakeDir(webDirectory || targetDirectory)
      If rc<>0
        then do
          Call lineout , 'ERROR - the specified target directory, "'webDirectory || targetDirectory'",' ,
                         'could not be created. (Return code' rc 'from SysMkDir)'
          exit 6
        end
      targetDirectory = directory(webDirectory || targetDirectory)
    end
    else targetDirectory = target
end
If right(targetDirectory,1)<>'\'
  then targetDirectory = targetDirectory'\'
trash = directory(sourceDirectory)      /* Back to the source directory       */
  

/******************************************************************************/
/*  Set the remaining variables                                               */
/******************************************************************************/

/* Set the web relative directory                                             */
If installType='Server'
  then webRelativeDirectory = substr(targetDirectory,length(webDirectory)+1)
  else webRelativeDirectory = substr(targetDirectory,4)
  
If installType<>'Server'
  then webServer = 'file:///'slash(targetDirectory)
  
If right(webServer,1)<>'/'
  then webServer = webServer'/'
  
/* Set the protocol and name of the web server                                */
If installType='Server'
  then webDirectory = webServer || slash(webRelativeDirectory)
  else webDirectory = webServer

If installType='Server'
  then NQdirectory = webServer || slash(webRelativeDirectory) || 'html/'
  else NQdirectory = slash(targetDirectory)'html'  


/******************************************************************************/
/*  At last we are ready to start the installation                            */
/******************************************************************************/

/* Find out if there is already an information center installed               */
If ''<>stream(targetDirectory'uninstall.rex','C','Q EXIST')
  then do
    Call lineout , 'There is already an Information Center installed at' targetDirectory'.'
    If 'Y'=GetYesNo('Do you want me to uninstall it?')
      then targetDirectory'uninstall.rex'
  end

/* Make sure the target directory is empty or can be overwritten              */
file. = fileTree(targetDirectory'*.*','O')
If file.0<>0
  then do
    Call lineout , 'There are already files or directories in' targetDirectory'.'
    If 'Y'=GetYesNo('Is it safe to erase them?')
      then call ClearDirectory targetDirectory
      else if 'N'=GetYesNo('Shall I carry on with the installation anyway?')
        then exit 7
  end

/* Copy most things from the source to the target                             */
call copyFile sourceDirectory'*.*', truncate(targetDirectory,1)
If installType<>'Minimum'
  then do
    Call copyDirectory sourceDirectory'html\', targetDirectory'html\'
    Call copyDirectory sourceDirectory'pdf\',  targetDirectory'pdf\'
  end
'ERASE' targetDirectory || 'install.rex'

/* Set up the NetQuestion indexes                                             */
imninstsrv = value('IMNINSTSRV',,'ENVIRONMENT')
If imninstsrv=''
  then do
    Call lineout , 'NetQuestion has not been installed or the system has not been'
    Call lineout , 'rebooted since installation so I cannot configure the NetQuestion'
    Call lineout , 'control files.'
    indexSuffix = ''
    NQserver = ''
  end
  else call SetupNQ

/* Set the definitions of the installation variabales in install.js           */
Call stringreplace targetDirectory'install.js' ,
                 , 'var webRelativeDirectory = "";' ,
                 , 'var webRelativeDirectory = "'slash(webRelativeDirectory)'";' ,
                 , 'var NQserver = "";' ,
                 , 'var NQserver = "'NQserver'";' ,
                 , 'var indexSuffix = "0";' ,
                 , 'var indexSuffix = "'indexSuffix'";'

If imninstsrv<>''
  then do
    /* Copy the header and footer for search results to the NetQuestion       */
    /* directory.  This should not be necessary if only we could find out     */
    /* how to make NetQuestion honour PASS directives in its configuration    */
    /* file.                                                                  */
    search = imninstsrv'\'webRelativeDirectory'html\search\'
    rc = MakeDir(search)
    call copyFile sourceDirectory'html\search\srchhead.html', search
    call copyFile sourceDirectory'html\search\srchfoot.html', search
    /* Modify srchhead.html to point to the main Information Center files     */
    Call stringreplace search'srchhead.html' ,
                     , 'var webDirectory = "";' ,
                     , 'var webDirectory = "'webDirectory'";'

    /* Copy the stars                                                         */
    call copyFile sourceDirectory'html\images\dfhstar*.gif', search

    /* Stop and restart the NetQuestion SBCS server so that it picks up the   */
    /* new master file and make sure that the HTTPDL server is running        */
    imninstsrv'\imnss.exe stop server'
    imninstsrv'\imnss.exe start server'
    imninstsrv'\nqdetach.exe' imninstsrv'\httpdl.exe -r' imninstsrv'\httpd.cnf'

    Call lineout , 'Please ensure that your browser is set to avoid the use of a proxy'
    Call lineout , 'server for the NetQuestion server' NQserver'.'
    
    /* If this is a Windows 95 system, add two icons to the startup folder    */
    Parse source opsys .
    If opsys='Windows95' | opsys='Windows98'
      then do
        pm = .WindowsProgramManager~new()
        if 0<>pm~InitCode()
          then do
            Call lineout , 'ERROR - unable to add icons to startup folder.  Please issue'
            Call lineout , 'the following commands after each startup:'
            Call lineout , ' '
            Call lineout , '    ' imninstsrv'\imnss.exe start server'
            Call lineout , '    ' imninstsrv'\nqdetach.exe' imninstsrv'\httpdl.exe -r' imninstsrv'\httpd.cnf'
          end
          else do
            Call lineout , 'Adding icons to the startup folder ...'
            windir = value('windir',,'ENVIRONMENT')
            If windir='' then windir = 'C:\WINDOWS'
            startUpFolder = windir'\Start Menu\Programs\StartUp'
            If ''=stream(startUpFolder'\Start NetQuestion search service.lnk','C','Q EXISTS')
              then pm~AddShortCut( startUpFolder'\Start NetQuestion search service' ,
                                 , '.\imnss.exe' ,
                                 , ,
                                 , ,
                                 , imninstsrv ,
                                 , 'start server' ,
                                 , ,
                                 , 'MINIMIZED' ,
                                 )
            If ''=stream(startUpFolder'\Start NetQuestion http server.lnk','C','Q EXISTS')
              then pm~AddShortCut( startUpFolder'\Start NetQuestion http server' ,
                                 , '.\nqdetach.exe' ,
                                 , ,
                                 , ,
                                 , imninstsrv ,
                                 , 'httpdl.exe -r httpd.cnf' ,
                                 , ,
                                 , 'MINIMIZED' ,
                                 )
          end
      end

  end


/* All done                                                                   */
Call lineout , ''
Call lineout , 'Installation of the CICS Information Center is complete.'

return

/******************************************************************************/
/* Initialise the NetQuestion indexes                                         */
/******************************************************************************/
SetupNQ:
    /* Copy the cgi-bin executables if necessary                              */
    if ''=stream(imninstsrv'\dfhsrsen.exe','C','Q EXIST')
      then call copyFile 'nq\cgi\dfhsrs*.exe', imninstsrv
      
    /* Get the name of the NetQuestion server                                 */
    If installType='Server'
      then NQserver = truncate(webServer,1)
      else NQserver = 'http://localhost'

    /* Add the port number, if necessary                                      */
    If lastpos(':',NQserver)<8
      then do
        If portNumber=''
          then portNumber = GetReply('Please enter the port number for server "'NQserver'"' ,
                                     'if it is not 49213:')
        If portNumber='' then portNumber = '49213'
        NQserver = NQserver':'portNumber
      end

    /* Update IMNMASTR.DAT and IMNMAP.DAT                                     */
    indexSuffix = nextIndex()
    If installType<>'Minimum' & indexSuffix=' '
      then indexSuffix = '0'
    Call UpdateNQ indexSuffix

    return

/******************************************************************************/
/* This contorted installation process allows multiple version of the         */
/* Information Center to coexist with the same NetQuestion server.  (It may   */
/* also be possible to achieve this using different instances of the server   */
/* listening on different ports but there would still have been a problem     */
/* with the search header and footer files.)  In order to allow for           */
/* different search indexes, the name of each search index is generated with  */
/* precisely seven characters which are then augmented by a single suffix     */
/* character.  This routine determines the next available suffix by examining */
/* the current NetQuestion master file.                                       */
/******************************************************************************/
nextIndex:
    /* Set the list of possible suffix characters                             */
    numalph = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    /* Scan the master file looking at the last character of each index name. */
    mapdata = queueCommand('imndomap -a')
    /* Erase the index characters that have been used                         */
    do while mapdata<>''
      Parse var mapdata 'Index ' indexName ' ' mapdata
      If indexName='' then leave
      If left(indexName,3)='DFH' & length(indexName)=8
        then do
          suffix = substr(indexName,8,1)
          If suffix<>' '
            then do
              p = pos(suffix,numalph)
              If p<>0 then numalph = overlay(' ',numalph,p)
            end
        end
    end
    /* Now look for the longest gap of unused index characters                */
    maxgap = 0
    startmaxgap = 0
    firstblank = verify(numalph,' ','M')
    i = firstblank
    do forever
      j = verify(numalph,' ',,i+1)     /* Find the next non-blank             */
      If j=0 then leave                /* Get out if they're all blank        */
      i = verify(numalph,' ','M',j+1)  /* Now find the next blank             */
      If i=0 then leave                /* Get out if there aren't any         */
      if i-j>maxgap                    /* If this gap is larger than previous */
        then do
          startmaxgap = j              /* Remember where it started           */
          maxgap = i-j                 /*   and remember how long it is       */
        end
    end
    If j=0                             /* If the last character is a blank    */
      then if firstblank-1>maxgap      /*   check the initial gap             */
        then startmaxgap = 1
        else nop
      else if firstblank+length(numalph)-j>maxgap
        then startmaxgap = j
        else nop
    return substr(numalph,startmaxgap,1)

/******************************************************************************/
/*                                                                            */
/* Update the NetQuestion map and master file with the new indexes            */
/*                                                                            */
/******************************************************************************/
updateNQ:
    Parse arg indexSuffix .
    /* Make sure the server is running                                        */
    trash = queueCommand(imninstsrv'\imnss.exe start server')
    
    /* Read the map file from the CD                                          */
    mapName = sourceDirectory'html\search\imnmap.dat'
    mapStream = .stream~new(mapName)

    /* Create the directory to hold the NetQuestion indexes                   */
    rc = MakeDir(targetDirectory'search')

    /**************************************************************************/    
    /* Add each index to the master files and copy the index files            */
    /**************************************************************************/    
    
    /* Start building a string to be added to the uninstall command           */
    uninstallString = 'Say ''Deleting NetQuestion indexes ...''' || '0D0A'x
    
    do while mapStream~lines()>0

      /* Get the name and title of the next index                             */
      index = mapStream~linein()
      Parse var index '"' indexName '"' '"' '"' '"' indexTitle '"'
      indexName = left(indexName,7) || indexSuffix
      Say ''
      Say 'Creating NetQuestion index' indexName
      
      /* Get the four character prefix from the index name                    */
      indexPrefix = lower(substr(indexName,4,4))
      
      /* Add the index to the map file                                        */
      trash = queueCommand('imndomap -d' indexName)
      'imndomap -c' indexName NQdirectory '"'indexTitle'"'
                    
      /* Add the index to the master file                                     */
      trash = queueCommand('imnixdel' indexName)
      'imnmtupd' '-m' imninstsrv ,
                 '-n' indexName ,
                 '-i' TargetDirectory'search\'indexPrefix'data' ,
                 '-w' TargetDirectory'search\'indexPrefix'work'

      /* Copy the index files, modifying the index name as we go              */
      dataDirectory = 'search\'indexPrefix'data\'
      rc = MakeDir(targetDirectory || dataDirectory)
      Say 'Copying NetQuestion files from' sourceDirectory || dataDirectory ,
          'to' targetDirectory || dataDirectory
      file. = fileTree(sourceDirectory || dataDirectory'*.*','FO')
      do f=1 to file.0
        Say '...' file.f
        
        /* Read the index file                                                */
        file = .stream~new(file.f)
        data = file~charin(1,file~query('SIZE'))
        file~close()
        
        /* Modify the data if it contains the index name                      */
        p = pos(left(indexName,7),left(data,320))
        If p>0
          then data = overlay(indexSuffix,data,p+7)
          
        /* Write the file to the target location                              */
        file = .stream~new(targetDirectory || dataDirectory || filespec('NAME',file.f))
        file~open('WRITE')
        file~charout(data)
        file~close()
      end

      /* Add the index to the list of indexes to be uninstalled               */
      uninstallString = uninstallString ,
                     || 'Call DeleteIndex' indexName || '0D0A'x
    end

    /* Add the name of the NetQuestion directory                              */
    comment = left('/* Set the name of the NetQuestion directory to be cleared',78)'*/'
    uninstallstring = uninstallstring ,
                   || '0D0A'x ,
                   || comment || '0D0A'x ,
                   || 'imninst = "'imninstsrv'\'webRelativeDirectory'"' || '0D0A'x

    /**************************************************************************/
    /* Complete the uninstall command                                         */
    /**************************************************************************/
    
    /* Read the prototype command                                             */
    uninstallCommandName = targetDirectory'uninstall.rex'
    uninstallFile = .stream~new(uninstallCommandName)
    uninstallCommand = uninstallFile~charin(1,uninstallFile~query('SIZE'))
    uninstallFile~close()
    
    /* Modify the command by adding the invocations of deleteIndex            */
    p = pos('/* Remove any NetQuestion indexes',uninstallCommand)
    If p=0
      then say 'Error - unable to complete modification of uninstall command'
      else do
        p = pos('0D0A'x,uninstallCommand,p)
        uninstallCommand = insert(uninstallString,uninstallCommand,p+1)
        /* Rewrite the modified command                                       */
        uninstallFile~open('WRITE REPLACE')
        uninstallFile~charout(uninstallCommand)
        uninstallFile~close()
      end
    
    /**************************************************************************/    
    /* Remove all the unnecessary data and work directories                   */
    /**************************************************************************/    
    Call removeDirectory imninstsrv || '\dfhsearch'

    return


/******************************************************************************/
/*                                                                            */
/*              Required classes and general purpose public routines          */
/*                                                                            */
/******************************************************************************/

::requires winsystm.cls

  /****************************************************************************/
  /* Get a "Yes" or a "No" reply to a question                                */
  /****************************************************************************/
  ::routine GetYesNo public
    Use arg prompt
    If prompt<>''
      then call lineout , prompt
    Call lineout , ''
    Call charout , '     '
    do until pos(reply,'YN')>0
      Parse pull reply
      reply = translate(left(reply,1))
      If pos(reply,'YN')=0
        then call charout , 'Please reply Yes or No: '
    end
    Call lineout , ''
    return reply

  /****************************************************************************/
  /* Get a reply to a question                                                */
  /****************************************************************************/
  ::routine GetReply public
    Use arg prompt
    If prompt<>''
      then call lineout , prompt
    Call lineout , ''
    Call charout , '     '
    Parse pull reply
    Call lineout , ''
    return reply

  /****************************************************************************/
  /* getParameter: get a parameter from the option string                     */
  /****************************************************************************/
  ::routine GetParameter public
    Use arg options, parameterName
    parameterName = '/' || translate(parameterName) || '='
    p = pos(parameterName,translate(options))
    If p=0 then return ""
    parameter = substr(options,p+length(parameterName))
    If pos(left(parameter,1),'''"')>0
      then parameter = left(parameter,pos(left(parameter,1),parameter,2))
      else parse var parameter parameter .
    return parameter
    
  /****************************************************************************/
  /* Copy the contents of a directory and all subdirectories.  This is        */
  /* equivalent to "XCOPY /e" but it should work on Windows 95 as well as     */
  /* on Windows NT.  (I had a problem that XCOPY crashed Windows 95,          */
  /* presumably because it was copying so many files.)                        */
  /****************************************************************************/
  ::routine CopyDirectory public
    Use arg sourceDirectory, targetDirectory
    Call MakeDir targetDirectory
    file. = fileTree(sourceDirectory'*.*','FO')
    If file.0>0
      then call copyFile sourceDirectory'*.*', truncate(targetDirectory,1)
    dir. = fileTree(sourceDirectory'*','DO')
    do d=1 to dir.0
      Call CopyDirectory dir.d'\', targetDirectory||filespec('name',dir.d)'\'
    end
    return
    
  /****************************************************************************/
  /* Copy a file                                                              */
  /****************************************************************************/
  ::routine copyFile public
    use arg sourceFileName, targetFileName
    file. = fileTree(sourceFileName,'FO')
    If file.0=0
      then say 'ERROR - file' sourceFileName 'does not exist'
      else 'COPY' sourceFileName targetFileName
    return
    
  /****************************************************************************/
  /* Remove a directory and all its contents                                  */
  /****************************************************************************/
  ::routine RemoveDirectory public
    Use arg directoryName
    rc = ClearDirectory(directoryName)
    rc = SysRmDir(directoryName)
    return 0
    
  /****************************************************************************/
  /* Remove all the contents of a directory and its subdirectories            */
  /****************************************************************************/
  ::routine ClearDirectory public
    Use arg directoryName
    dir. = fileTree(directoryName'\*.*','DO')
    do d=1 to dir.0
      rc = RemoveDirectory(dir.d)
    end
    rc = EraseAll(directoryName)
    return 0
    
  /****************************************************************************/
  /* Erase all the files in a directory                                       */
  /****************************************************************************/
  ::routine EraseAll public
    Use arg directoryName
    file. = fileTree(directoryName'\*','FO')
    do f=1 to file.0
      rc = SysFileDelete(file.f)
    end
    return 0
    
  /****************************************************************************/
  /* Truncate a string by a specified number of characters                    */
  /****************************************************************************/
  ::routine truncate public
    use arg string, length
    return left(string,length(string)-length)

  /****************************************************************************/
  /* Replace all occurrences of '\' by '/' in a specified string.             */
  /****************************************************************************/
  ::routine slash public
    Parse arg string
    return translate(string,'/','\')
    
  /****************************************************************************/
  /* Replace all occurrences of one string by another in the specified file.  */
  /****************************************************************************/
  ::routine stringreplace public
    Use arg filename
    file = .stream~new(filename)
    If ''=file~query('EXISTS')
      then do
        say 'ERROR - file' filename 'does not exist'
        return
      end
    data = file~charin(1,file~query('SIZE'))
    file~close()
    do a=2 by 2 to arg()
      data = gsub(data,arg(a),arg(a+1))
    end
    file~open('WRITE REPLACE')
    rc = file~charout(data)
    file~close()
    return

  /****************************************************************************/
  /* Replace all occurrences of one string by another in a specified string.  */
  /****************************************************************************/
  ::routine gsub public
    Parse arg string, old, new
    i = 0
    do forever
      i = pos(old,string,i+1)
      If i=0 then leave
      string = insert(new,delstr(string,i,length(old)),i-1)
      i = i+length(new)
    end
    return string

  /****************************************************************************/
  /* Convert a string to lower case                                           */
  /****************************************************************************/
  ::routine lower public
    Use arg string
    return translate(string,'abcdefghijklmnopqrstuvwxyz' ,
                           ,'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
                           
  /****************************************************************************/
  /*  Execute a command and return the input buffer                           */
  /****************************************************************************/
  ::routine queueCommand public
    use arg command
    /* Create a new buffer                                                    */
    newQueue = rxqueue('create')
    oldQueue = rxqueue('set', newQueue)
    /* Issue the command and read the input buffer                            */
    command '| RXqueue' newQueue
    inputBuffer = getQueue()
    /* Delete the buffer and restore the original buffer                      */
    call rxqueue 'Delete', newQueue
    call rxqueue 'Set', oldQueue
    /* Return the contents of the buffer                                      */
    return inputBuffer

  /****************************************************************************/
  /*  Concatenate the lines that are stacked in the input buffer              */
  /****************************************************************************/
  ::routine getQueue public
    queueData = ''
    do queued()
      parse pull line
      queueData = queueData || line || '0D0A'x
    end
    return queueData

  /****************************************************************************/
  /* Make a directory                                                         */
  /****************************************************************************/
  ::routine MakeDir public
    Use arg Directory
    If right(Directory,1)='\' then Directory = truncate(Directory,1)
    /* Check whether or not the directory already exists                      */
    dir. = fileTree(Directory,'DO')
    /* Return at once if the directory already exists                         */    
    If dir.0 >0
      then return 0
    /* Make sure the next higher directory exists                             */
    rc = MakeDir(Left(Directory, LastPos('\',Directory)-1))                    
    /* Finally we can create the directory                                    */
    Say 'Creating directory' Directory
    Say queueCommand('MKDIR' Directory)
    return rc

  /****************************************************************************/
  /* Look for files or directories                                            */
  /****************************************************************************/
  ::routine fileTree public
    use arg name, options
    rc = SysFileTree(name,'tree',options)
    If rc<>0
      then say 'ERROR - return code' rc 'from SysFileTree when looking for' name
    return tree.
        
/******************************************************************************/
/*  End of file                                                               */
/******************************************************************************/

